@@ -2,7 +2,7 @@
   FUSE: Filesystem in Userspace
   Copyright (C) 2001-2007  Miklos Szeredi <miklos@szeredi.hu>
 
-  This program can be distributed under the terms of the GNU GPLv2.
+  This program can be distributed under the terms of the GNU GPL.
   See the file COPYING.
 */
 
@@ -23,25 +23,6 @@
 #include <sys/syscall.h>
 #include <linux/capability.h>
 #include <linux/securebits.h>
-/* for 2.6 kernels */
-#if !defined(SECBIT_KEEP_CAPS) && defined(SECURE_KEEP_CAPS)
-#define SECBIT_KEEP_CAPS (issecure_mask(SECURE_KEEP_CAPS))
-#endif
-#if !defined(SECBIT_KEEP_CAPS_LOCKED) && defined(SECURE_KEEP_CAPS_LOCKED)
-#define SECBIT_KEEP_CAPS_LOCKED (issecure_mask(SECURE_KEEP_CAPS_LOCKED))
-#endif
-#if !defined(SECBIT_NO_SETUID_FIXUP) && defined(SECURE_NO_SETUID_FIXUP)
-#define SECBIT_NO_SETUID_FIXUP (issecure_mask(SECURE_NO_SETUID_FIXUP))
-#endif
-#if !defined(SECBIT_NO_SETUID_FIXUP_LOCKED) && defined(SECURE_NO_SETUID_FIXUP_LOCKED)
-#define SECBIT_NO_SETUID_FIXUP_LOCKED (issecure_mask(SECURE_NO_SETUID_FIXUP_LOCKED))
-#endif
-#if !defined(SECBIT_NOROOT) && defined(SECURE_NOROOT)
-#define SECBIT_NOROOT (issecure_mask(SECURE_NOROOT))
-#endif
-#if !defined(SECBIT_NOROOT_LOCKED) && defined(SECURE_NOROOT_LOCKED)
-#define SECBIT_NOROOT_LOCKED (issecure_mask(SECURE_NOROOT_LOCKED))
-#endif
 #endif
 
 #include "fuse.h"
@@ -233,7 +214,6 @@ int main(int argc, char *argv[])
 {
 	char *type = NULL;
 	char *source;
-	char *dup_source = NULL;
 	const char *mountpoint;
 	char *basename;
 	char *options = NULL;
@@ -243,9 +223,7 @@ int main(int argc, char *argv[])
 	int dev = 1;
 	int suid = 1;
 	int pass_fuse_fd = 0;
-	int fuse_fd = 0;
 	int drop_privileges = 0;
-	char *dev_fd_mountpoint = NULL;
 
 	progname = argv[0];
 	basename = strrchr(argv[0], '/');
@@ -343,7 +321,6 @@ int main(int argc, char *argv[])
 				}
 				opt = strtok(NULL, ",");
 			}
-			free(opts);
 		}
 	}
 
@@ -364,8 +341,7 @@ int main(int argc, char *argv[])
 
 	if (!type) {
 		if (source) {
-			dup_source = xstrdup(source);
-			type = dup_source;
+			type = xstrdup(source);
 			source = strchr(type, '#');
 			if (source)
 				*source++ = '\0';
@@ -403,7 +379,7 @@ int main(int argc, char *argv[])
 #endif
 
 		struct passwd *pwd = getpwnam(setuid_name);
-		if (!pwd || setgid(pwd->pw_gid) == -1 || setuid(pwd->pw_uid) == -1) {
+		if (setgid(pwd->pw_gid) == -1 || setuid(pwd->pw_uid) == -1) {
 			fprintf(stderr, "%s: Failed to setuid to %s: %s\n",
 				progname, setuid_name, strerror(errno));
 			exit(1);
@@ -414,8 +390,8 @@ int main(int argc, char *argv[])
 	}
 
 	if (pass_fuse_fd)  {
-		fuse_fd = prepare_fuse_fd(mountpoint, type, options);
-		dev_fd_mountpoint = xrealloc(NULL, 20);
+		int fuse_fd = prepare_fuse_fd(mountpoint, type, options);
+		char *dev_fd_mountpoint = xrealloc(NULL, 20);
 		snprintf(dev_fd_mountpoint, 20, "/dev/fd/%u", fuse_fd);
 		mountpoint = dev_fd_mountpoint;
 	}
@@ -434,17 +410,8 @@ int main(int argc, char *argv[])
 		add_arg(&command, options);
 	}
 
-	free(options);
-	free(dev_fd_mountpoint);
-	free(dup_source);
-	free(setuid_name);
-
 	execl("/bin/sh", "/bin/sh", "-c", command, NULL);
 	fprintf(stderr, "%s: failed to execute /bin/sh: %s\n", progname,
 		strerror(errno));
-	
-	if (pass_fuse_fd)
-		close(fuse_fd);
-	free(command);
 	return 1;
 }
